! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!+
! Module: elcbuild_mod
!
! Authors:  Didier BARRET - CESR
!          
! Description: Contruct time series for lcplot
! 
! Comments: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module elcbuild_mod

  use dal
  use param  
  use errorHandling

  implicit none 
  
  private 

  public elcbuild

contains 
  
  !---------------------------------------------------------------------
  subroutine elcbuild
  !---------------------------------------------------------------------

  character (len=4096), pointer :: inset(:)
  character (len=4096) :: outset

  integer :: nb_lc,i,nRateRef,nRate

  real(kind=single) :: EminTemp, EmaxTemp

  integer :: s(1) 

  type(DataSetT) :: RateSet, OutRateSet
  type(TableT) :: RateTable, OutRateTable
  type(ColumnT) :: ColRate, ColRateE, ColBackV, ColBackE, ColFracExp
  type(ColumnT) :: OutColRate, OutColRateE, OutColBackV, OutColBackE, OutColFracExp

  real(kind=single), pointer :: ValColRate(:),ValColRateE(:),ValColBackV(:)
  real(kind=single), pointer :: ValColBackE(:), ValColFracExp(:)

  real(kind=SINGLE), dimension(:,:), pointer :: OutValColRate,OutValColRateE,OutValColBackV
  real(kind=SINGLE), dimension(:,:), pointer :: OutValColBackE, OutValColFracExp

! Taking timezero out of elcbuild, as it is not given in the headers of the 
! new lccorr (2/5/2001)

!  real(kind=DOUBLE)::TimezeroRef, TimedelRef,TstartRef,TstopRef
!  real(kind=DOUBLE)::Timezero, Timedel,Tstart,Tstop

   real(kind=DOUBLE)::TimedelRef,TstartRef,TstopRef
   real(kind=DOUBLE)::Timedel,Tstart,Tstop

  nb_lc = parameterCount("sets")

  allocate(inset(1:nb_lc))

  if (nb_lc < 2) then
      call error ("InsufficientInput","The number of input time series must be larger than 2 !")
  end if 

  do i = 1,nb_lc 
     inset(i)=trim(stringParameter("sets",i-1))
  end do


  outset=StringParameter("outset")

  RateSet=dataSet(inset(1), read)
  RateTable = Table(RateSet,"RATE")
  nRateRef = numberOfRows(RateTable)
!  TimezeroRef = real64Attribute(RateTable, "TIMEZERO")
  TimedelRef = real64Attribute(RateTable, "TIMEDEL")
  TstartRef = real64Attribute(RateTable, "TSTART")
  TstopRef = real64Attribute(RateTable, "TSTOP")

! That is where I check that all the input light curves cover the same time 
! intervals

  do i = 2,nb_lc
     RateSet=dataSet(inset(i), read)
     RateTable = Table(RateSet,"RATE")

     nRate = numberOfRows(RateTable)
!     Timezero = real64Attribute(RateTable, "TIMEZERO")
     Timedel = real64Attribute(RateTable, "TIMEDEL")
     Tstart = real64Attribute(RateTable, "TSTART")
     Tstop = real64Attribute(RateTable, "TSTOP")

!     If (Timezero /= TimezeroRef .OR. Tstart /= TstartRef .OR. &
!        Tstop /= TstopRef .OR. Timedel /= TimedelRef .OR. nRate /= nRateRef) then 
!        call error("difftimeintervals","TIMEZERO, TIMEDEL, TSTART, TSTOP must be the same for all files")
!     endif

!  end do

! If I have reached that point, then I can proceed and create the output time series

     If (Tstart /= TstartRef .OR. &
        Tstop /= TstopRef .OR. Timedel /= TimedelRef .OR. nRate /= nRateRef) then 
        call error("difftimeintervals","TIMEDEL, TSTART, TSTOP must be the same for all files")
     endif

  end do

  OutRateSet=DataSet(outset,CREATE)

  call setAttributes(attributable(outRateSet), attributable(RateSet))

  outRateTable = addTable(outRateSet, "RATE",nRateRef)

  call setAttributes(attributable(outRateTable), attributable(RateTable))

  s=nb_lc

  OutColRate=AddColumn(outRateTable,"RATE",real32,"COUNTS/S",s)
  OutColRateE=AddColumn(outRateTable,"ERROR",real32,"COUNTS/S",s)
  OutColBackV=AddColumn(outRateTable,"BACKV",real32,"COUNTS/S",s)
  OutColBackE=AddColumn(outRateTable,"BACKE",real32,"COUNTS/S",s)
  OutColFracExp=AddColumn(outRateTable,"FRACEXP",real32," ",s)


  OutValColRate => real32VectorData(OutColRate)
  OutValColRateE =>  real32VectorData(OutColRateE)
  OutValColBackV => real32VectorData(OutColBackV)
  OutValColBackE =>  real32VectorData(OutColBackE)
  OutValColFracExp =>  real32VectorData(OutColFracExp)

  call setAttribute(outRateTable,"NUMBAND",nb_lc)

  do i = 1,nb_lc

     RateSet=dataSet(inset(i), read)
     RateTable = Table(RateSet,"RATE")

!     EminTemp=real32Attribute(RateTable,'E_MIN')
!     EmaxTemp=real32Attribute(RateTable,'E_MAX')
!     EunitTemp=StringAttribute(RateTable,'EUNIT')

!     call setAttribute(outRateTable,"E_MIN"//int2asc(i), EminTemp)
!     call setAttribute(outRateTable,"E_MAX"//int2asc(i), EmaxTemp)
!     call setAttribute(outRateTable,"EUNIT"//int2asc(i), trim(EunitTemp))

     ColRate=Column(RateTable,"RATE",read)
     ValColRate => real32data(ColRate)
     OutValColRate(i-1,:) = ValColRate

     ColRateE=column(RateTable,"ERROR",read)
     ValColRateE => real32data(ColRateE)
     OutValColRateE(i-1,:) = ValColRateE


     ColBackV=column(RateTable,"BACKV",read)
     ValColBackV => real32data(ColBackV)
     OutValColBackV(i-1,:) = ValColBackV
   
     ColBackE=column(RateTable,"BACKE",read)
     ValColBackE => real32data(ColBackE)
     OutValColBackE(i-1,:) = ValColBackE

     ColFracExp=column(RateTable,"FRACEXP",read)
     ValColFracExp => real32data(ColFracExp)
     OutValColFracExp(i-1,:) = ValColFracExp
     call release(RateSet)
  end do
  deallocate(inset)
  call release(OutRateSet)

  end subroutine elcbuild

  !-------------------------------------------------------------------------------------
  function int2asc(nb0) result(str)
  !-------------------------------------------------------------------------------------
    integer,intent(in) :: nb0
    character(len=15) :: str

    integer :: i,lg,cp,lenstr,posi,deb,nb
    character(len=1) :: tmp
    
    deb = 1
    nb =nb0
    str = "               "
    if (nb == 0) then 
          str(1:1) = "0"
    else
       if (nb < 0) then 
          deb = 2
          str(1:1) = "-"
          nb = -nb
       end if
       lg = aint(log10(real(nb))) + deb
       lenstr = len(str)
       cp = nb
       do i=1,lg 
          tmp = achar(48+mod(cp,10))
          posi = lg-i+deb
          str(posi:posi) = tmp
          cp = cp / 10
       end do     
    end if
  end function int2asc

 
end module elcbuild_mod
